"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getFilesWithMetadata = exports.prepPayForFilesInstructions = exports.uploadToArweave = exports.LAMPORT_MULTIPLIER = exports.MEMO_ID = exports.ARWEAVE_UPLOAD_URL = exports.AR_SOL_HOLDER_ID = void 0;
const web3_js_1 = require("@solana/web3.js");
const crypto_1 = __importDefault(require("crypto"));
exports.AR_SOL_HOLDER_ID = new web3_js_1.PublicKey("HvwC9QSAzvGXhhVrgPmauVwFWcYZhne3hVot9EbHuFTm");
exports.ARWEAVE_UPLOAD_URL = process.env.REACT_APP_ARWEAVE_UPLOAD_URL || "https://us-central1-principal-lane-200702.cloudfunctions.net/uploadFileProd2";
exports.MEMO_ID = new web3_js_1.PublicKey('MemoSq4gqABAXKb96qnH8TysNcWxMyWCqXgDLGmfcHr');
exports.LAMPORT_MULTIPLIER = Math.pow(10, 9);
const WINSTON_MULTIPLIER = Math.pow(10, 12);
function getAssetCostToStore(files) {
    return __awaiter(this, void 0, void 0, function* () {
        const totalBytes = files.reduce((sum, f) => (sum += f.size), 0);
        console.log("Total bytes", totalBytes);
        const txnFeeInWinstons = parseInt(yield (yield fetch("https://arweave.net/price/0")).text());
        console.log("txn fee", txnFeeInWinstons);
        const byteCostInWinstons = parseInt(yield (yield fetch("https://arweave.net/price/" + totalBytes.toString())).text());
        console.log("byte cost", byteCostInWinstons);
        const totalArCost = (txnFeeInWinstons * files.length + byteCostInWinstons) / WINSTON_MULTIPLIER;
        console.log("total ar", totalArCost);
        let conversionRates = JSON.parse(localStorage.getItem("conversionRates") || "{}");
        if (!conversionRates ||
            !conversionRates.expiry ||
            conversionRates.expiry < Date.now()) {
            console.log("Calling conversion rate");
            conversionRates = {
                value: JSON.parse(yield (yield fetch("https://api.coingecko.com/api/v3/simple/price?ids=solana,arweave&vs_currencies=usd")).text()),
                expiry: Date.now() + 5 * 60 * 1000,
            };
            if (conversionRates.value.solana)
                localStorage.setItem("conversionRates", JSON.stringify(conversionRates));
        }
        // To figure out how many lamports are required, multiply ar byte cost by this number
        const arMultiplier = conversionRates.value.arweave.usd / conversionRates.value.solana.usd;
        console.log("Ar mult", arMultiplier);
        // We also always make a manifest file, which, though tiny, needs payment.
        return exports.LAMPORT_MULTIPLIER * totalArCost * arMultiplier * 1.1;
    });
}
function uploadToArweave(txid, mintKey, files, uploadUrl = exports.ARWEAVE_UPLOAD_URL) {
    var _a, _b;
    return __awaiter(this, void 0, void 0, function* () {
        // Ship it off to ARWeave!
        const data = new FormData();
        const tags = files.reduce((acc, f) => {
            acc[f.name] = [{ name: "mint", value: mintKey.toBase58() }];
            return acc;
        }, {});
        data.append("tags", JSON.stringify(tags));
        data.append("transaction", txid);
        // data.append("env", IS_DEV ? "devnet" : "mainnet-beta");
        files.map((f) => data.append("file[]", f));
        // TODO: convert to absolute file name for image
        try {
            const result = yield (yield fetch(
            // TODO: add CNAME
            uploadUrl, {
                method: "POST",
                body: data,
            })).json();
            return result;
        }
        catch (e) {
            if ((_b = (_a = e.response) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.message) {
                throw new Error(e.response.data.message);
            }
            throw e;
        }
    });
}
exports.uploadToArweave = uploadToArweave;
const prepPayForFilesInstructions = (payer, files) => __awaiter(void 0, void 0, void 0, function* () {
    const memo = exports.MEMO_ID;
    const instructions = [];
    instructions.push(web3_js_1.SystemProgram.transfer({
        fromPubkey: payer,
        toPubkey: exports.AR_SOL_HOLDER_ID,
        lamports: yield getAssetCostToStore(files),
    }));
    for (let i = 0; i < files.length; i++) {
        const hashSum = crypto_1.default.createHash("sha256");
        hashSum.update(yield files[i].text());
        const hex = hashSum.digest("hex");
        instructions.push(new web3_js_1.TransactionInstruction({
            keys: [],
            programId: memo,
            data: Buffer.from(hex),
        }));
    }
    return instructions;
});
exports.prepPayForFilesInstructions = prepPayForFilesInstructions;
function getFilesWithMetadata(files, metadata) {
    var _a;
    const metadataContent = {
        name: metadata.name,
        symbol: metadata.symbol,
        description: metadata.description,
        seller_fee_basis_points: metadata.sellerFeeBasisPoints,
        image: metadata.image,
        animation_url: metadata.animation_url,
        external_url: metadata.external_url,
        properties: Object.assign(Object.assign({}, metadata.properties), { creators: (_a = metadata.creators) === null || _a === void 0 ? void 0 : _a.map((creator) => {
                return {
                    address: creator.address,
                    share: creator.share,
                };
            }) }),
    };
    return [
        ...files,
        new File([JSON.stringify(metadataContent)], "metadata.json"),
    ];
}
exports.getFilesWithMetadata = getFilesWithMetadata;
//# sourceMappingURL=arweave.js.map